<?php

include('IXR_Library.inc.php');
include('config.inc.php');


$err = "";

function okmsg($text)
{
	return "$text\n\n" .
			"Thanks! Visit http://dekstop.de/ilike/ to see what else people listen to.";
}

function errmsg($text, $args)
{
	return "Server error: $text Please contact me at http://dekstop.de/contact/ if this problem persists.\n\n".
		date("m.d. y H:i:s") ."\nargs = ['". implode("', '", $args) ."']";
}

function isValidAgent($userAgent)
{
	$t = str_replace ($forbidden, $replacements, $userAgent);
	if (strpos($t, "musikCube/iLikeThis 0.1") != FALSE || 
		strpos($t, "musikCube/iLikeThis 0.2") != FALSE)
		return false;
	else
		return true;
}

// returns TRUE on success, FALSE on error.
// if it fails: sets $err to an error message string.
function appendToFile($filename, $line) 
{

	global $err;
	
	$err = "";	
	
	if (is_writable($filename)) 
	{	
	
		if (!$handle = fopen($filename, 'a')) 
		{
			$err = "Cannot open file $filename.";
			return FALSE;
		}
		if (fwrite($handle, $line."\n") === FALSE) 
		{
			$err =  "Cannot write to file $filename.";
			return FALSE;
		}
	
		fclose($handle);
		return TRUE;
	
	} else 
	{
		$err = "The file $filename is not writable.";
		return FALSE;
	}
}

// creates a tab-separated string from an array of strings
// and returns this string
function createTabSeparatedLine($strings)
{
	global $storeIP;
	
	$forbidden = array("\t", "\n");
	$replacements = array("", "");
	
	if ($storeIP == true)
	{
		$line = $_SERVER["REMOTE_ADDR"] . "\t" . time() . "\t" .
			implode("\t", str_replace ($forbidden, $replacements, $strings));
	}
	else
	{
		$line = "" . "\t" . time() . "\t" .
			implode("\t", str_replace ($forbidden, $replacements, $strings));
	}
	return $line;
}


/* Functions defining the behaviour of the server */

// $args[0] - user agent
// $args[1] - comment
// $args[2] - artist name
// $args[3] - album
// $args[4] - title
function addSong($args) 
{
	
	global $err, $dbdir, $songfile;
	
	if (!isValidAgent($args[0]))
		return "You need a newer version of this plugin. Get it at http://dekstop.de/ilike/";
	
	if (count($args) != 5)
		return errmsg("Wrong amount of parameters.", $args);
	
	if (appendToFile($dbdir.$songfile, createTabSeparatedLine($args)))
		return okmsg("Added song '$args[2] - $args[4]' to the recommendations.");
	else
		return errmsg($err, $args);
}


// $args[0] - user agent
// $args[1] - comment
// $args[2] - artist name
// $args[3] - album
function addAlbum($args) 
{
	global $err, $dbdir, $albumfile;
	
	if (!isValidAgent($args[0]))
		return "You need a newer version of this plugin. Get it at http://dekstop.de/ilike/";
	
	if (count($args) != 4)
		return errmsg("Wrong amount of parameters.", $args);
	
	if (appendToFile($dbdir.$albumfile, createTabSeparatedLine($args)))
		return okmsg("Added album '$args[2] - $args[3]' to the recommendations.");
	else
		return errmsg($err, $args);
}


// $args[0] - user agent
// $args[1] - comment
// $args[2] - netradio name
// $args[3] - stream url
// $args[4] - netradio homepage
function addNetRadio($args) 
{
	global $err, $dbdir, $netradiofile;
	
	if (!isValidAgent($args[0]))
		return "You need a newer version of this plugin. Get it at http://dekstop.de/ilike/";
	
	if (count($args) != 5)
		return errmsg("Wrong amount of parameters.", $args);
	
	if (appendToFile($dbdir.$netradiofile, createTabSeparatedLine($args)))
		return okmsg("Added netradio '$args[2]' to the recommendations.");
	else
		return errmsg($err, $args);
}

/* Create the server and map the XML-RPC method names to the relevant functions */

$server = new IXR_Server(array(
    'iLikeThis.song' => 'addSong',
    'iLikeThis.album' => 'addAlbum',
    'iLikeThis.netRadio' => 'addNetRadio'
));

?>